<?php

namespace Odhen\API\Service;

use \SimpleXMLElement;

class Extratocons {

    protected $entityManager;
    protected $instanceManager;
    protected $sessionHandler;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Helpers\Environment $sessionHandler){
        $this->entityManager = $entityManager;
        $this->instanceManager = $instanceManager;
        $this->sessionHandler = $sessionHandler;
    }

    public function consultaSaldo($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_SALDO',
            'P_CDFILIAL' => $CDFILIAL,
            'P_CDCLIENTE' => $CDCLIENTE,
            'P_CDCONSUMIDOR' => $CDCONSUMIDOR,
            'P_CDFAMILISALD' => $CDFAMILISALD,
            'P_DTEXTRATO' => ''
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        if (!empty($result) && !isset($result[0]['VRSALDCONFAM'])){
            $result = $result[0];
        }
        return $result;
    }

    public function consultaCartao($searchValue){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_CARTAO',
            'P_CDFILIAL' => '',
            'P_CDCLIENTE' => '',
            'P_CDCONSUMIDOR' => $searchValue,
            'P_CDFAMILISALD' => '',
            'P_DTEXTRATO' => ''
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        return $result;
    }

    public function consultaConsumer($searchValue, $CDFILIAL){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_CONSUMIDOR_ONLINE',
            'P_CDFILIAL' => $CDFILIAL,
            'P_CDCLIENTE' => '',
            'P_CDCONSUMIDOR' => $searchValue,
            'P_TCDCLIENTE' => 'T',
            'P_IDSITCONSUMI' => 'T',
            'P_CDFAMILISALD' => '',
            'P_DTEXTRATO' => ''
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        return $result;
    }

    public function consultaConsumoFamilia($CDCLIENTE, $CDCONSUMIDOR, $date){
        $params = array(
            'P_TPOPERACAO' => 'CONSULTA_CONSUMO_FAMILIA',
            'P_CDFILIAL' => '',
            'P_CDCLIENTE' => $CDCLIENTE,
            'P_CDCONSUMIDOR' => $CDCONSUMIDOR,
            'P_CDFAMILISALD' => '',
            'P_DTEXTRATO' => $date
        );
        $result = self::callExtratocons('RecuperaDados', $params);
        return $result;
    }

    public function insereExtratocons($params){
        $xml = new SimpleXMLElement('<ExtratoCons><Registro></Registro></ExtratoCons>');
        foreach ($params as $key => $value){
            $xml->Registro->addChild(strtolower($key), $value);
        }

        $params = array(
            'P_XMLEXTCONS' => $xml->asXml()
        );
        self::callExtratocons('InsereDados', $params);
    }

    public function consultaFidelidade($CPF, $FIDELIDADE='4ALL'){
        $params = array(
            "cpf" => $CPF
        );
        if ($FIDELIDADE=='UDS'){
            $result = self::callFidelityUDS('getSaldo', $params);
        } else {
            $result = self::callFidelity('user', $params);
        }
        if (!$result['success']){
            $result['userInfo'] = array(
                'cpf' => $CPF,
                'balance' => 0,
                'fidelidade' => $FIDELIDADE
            );
        }
        if(isset($result['bloqueado'])){
            if($result['bloqueado']){
                throw new \Exception("Usu�rio encontra-se bloqueado: <br>".$result['message']);
            }
        }
        return $result;
    }

    public function validaCupom($storeCode, $CPF4ALL, $CODPROMO, $SUBTOTAL, $productString){
        $params = array(
            "userId" => $CPF4ALL,
            "friendlyCode" => $CODPROMO,
            "storeReference" => $storeCode,
            "channel" => "pdv",
            "products" => $productString,
            "payment" => array(
                "subTotal" => $SUBTOTAL
            )
        );

        $result = self::callFidelity('validate', json_encode($params));

        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function transmiteVenda($vendaJSON, $FIDELIDADE='4ALL'){
        if ($FIDELIDADE=='UDS'){
            $result = self::callFidelityUDS('sendSale', $vendaJSON);
        } else {
            $result = self::callFidelity('order', $vendaJSON);
        }
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function transmitePrePedido($pedidoJSON, $FIDELIDADE='UDS'){
        if ($FIDELIDADE=='UDS'){
            $result = self::callFidelityUDS('sendOrder', $pedidoJSON);
        }
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function deletaPrePedido($pedidoJSON, $FIDELIDADE='UDS'){
        if ($FIDELIDADE=='UDS'){
            $result = self::callFidelityUDS('deleteOrder', $pedidoJSON);
        }
        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }
    
    public function cancelaFidelidade($orderId, $FIDELIDADE = '4ALL'){
        if($FIDELIDADE == '4ALL'){
            return self::cancela4ALL($orderId);
        } else if($FIDELIDADE == 'UDS') {
            return self::cancelaUDS($orderId);
        }
    }

    public function cancela4ALL($orderId){
        $params = array(
            "orderId" => $orderId
        );
        $result = $this::callFidelity('order/cancel', json_encode($params));

        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    public function cancelaUDS($vendaXML){
        $result = $this::callFidelityUDS('cancelSale', $vendaXML);

        if (!$result['success']) throw new \Exception("<br>".$result['message']);
        else return $result;
    }

    private function callExtratocons($method, $params){
        $postfields = "";
        foreach ($params as $key => $param){
            $postfields .= "&".$key."=".$param;
        }
        $postfields = substr($postfields, 1);

        $URL = $this->sessionHandler->getUserInfo()['CDURLWSEXTCONS'];
        $URL = rtrim($URL, '/') . "/";

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $URL.$method);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
        curl_setopt($curl, CURLOPT_POSTFIELDS, $postfields);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        if (!empty($this->instanceManager->getParameter('PROXY_IP'))){
            curl_setopt($curl, CURLOPT_PROXY, $this->instanceManager->getParameter('PROXY_IP').":".$this->instanceManager->getParameter('PROXY_PORT'));
            if (!empty($this->instanceManager->getParameter('PROXY_USER'))){
                curl_setopt($curl, CURLOPT_PROXYUSERPWD, $this->instanceManager->getParameter('PROXY_USER').":".$this->instanceManager->getParameter('PROXY_PASSWORD'));
            }
        }

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        curl_close($curl);

        if ($err){
            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer conex&atilde;o com o servidor do EXTRATOCONS ONLINE.");
            }
            else {
                throw new \Exception("<br>Ocorreu um erro na chamada do EXTRATOCONS ONLINE:<br><br>" . $errmsg);
            }
        }

        if ($method == 'RecuperaDados'){
            if (!self::isXml($curl_response)){
                throw new \Exception("<br>O EXTRATOCONS ONLINE retornou uma mensagem inesperada:<br><br>" . $curl_response);
            }
            else {
                $xml = simplexml_load_string($curl_response);
                $json_result = json_encode($xml);

                $index = 0;
                while (strpos($json_result, 'Registro') > -1){
                    $json_result = preg_replace(preg_quote('"Registro"', '/'), $index++, $json_result, 1);
                }

                $result = json_decode($json_result, true);

                if(!isset($result['ExtratoCons'])){
                    $result['ExtratoCons'] = $result;
                }

                return $result['ExtratoCons'];
            }
        }
        else if ($method == 'InsereDados' && $curl_response !== "OK"){
            throw new \Exception("<br>O EXTRATOCONS ONLINE retornou um erro:<br><br>" . $curl_response);
        }
    }

    private function callFidelity($method, $params){
        $URL = $this->sessionHandler->getUserInfo()['CDURLWSFID4ALL'];
        $URL = rtrim($URL, '/') . "/";
        $URL .= $method;
        if ($method === 'user') $URL .= '?'.http_build_query($params);

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $URL);

        if ($method === 'order' || $method === 'validate'){
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
        }
        if ($method === 'order/cancel'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'PUT');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
        }

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);

        $curl_response = curl_exec($curl);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        curl_close($curl);

        if ($err){
            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.");
            }
            else {
                throw new \Exception("<br>Ocorreu um erro na chamada com o servidor:<br><br>" . $errmsg);
            }
        }

        //$curl_response = '{"success":false,"message":"O orderId informado é inválido"}';
        //$curl_response = '{"success":false,"message":"Usuário não encontrado","userInfo":null}';
        //$curl_response = '{"success":true,"message":null,"userInfo":{"id":1127,"exist":true,"complete":true,"customerId":null,"blocked":false,"phone":"5551919976326","cpf":"66694211090","migrated":false,"marketplaceId":12,"birthdate":"1980-08-28","name":"Robin Hood","balance":1766,"category":{"name":"GREEN","type":"PERCENTAGE","cashbackValue":5,"expirationDate":"2021-04-11T23:59:59.000Z","color":"#84cec1"},"promocodes":[{"friendlyCode":"VALUE50","description":"","id":"5f92e6fede9a6326c7ef8b7f","gift":{"type":"value","value":5000},"expiration_date":"2021-10-30T02:00:00.000Z","status":"D"},{"friendlyCode":"KIELINGRANGE","description":"","id":"5f3d320685ff435adc965274","gift":{"type":"percentual","value":5000},"expiration_date":"2020-11-23T23:59:00.000Z","status":"D"},{"friendlyCode":"PERCENT20","description":"","id":"5fa1bd3ab28f216bdcaa9bd8","gift":{"type":"percentual","value":2000},"expiration_date":"2021-11-03T03:00:00.000Z","status":"D"},{"friendlyCode":"VALUE40","description":"","id":"5fa2b2b178fedf4c4837b67f","gift":{"type":"value","value":4000},"expiration_date":"2021-11-04T03:00:00.000Z","status":"D"},{"friendlyCode":"VALUE30","description":"","id":"5fa2bd1978fedf4c4837b682","gift":{"type":"value","value":3000},"expiration_date":"2021-10-30T02:00:00.000Z","status":"D"},{"friendlyCode":"percent100","description":"","id":"5fa2be7b78fedf4c4837b684","gift":{"type":"percentual","value":10000},"expiration_date":"2021-10-30T02:00:00.000Z","status":"D"},{"friendlyCode":"PERCENT100","description":"","id":"5fa2bf5478fedf4c4837b685","gift":{"type":"percentual","value":10000},"expiration_date":"2021-10-30T02:00:00.000Z","status":"D"},{"friendlyCode":"PERCENT110","description":"","id":"5fa2bf9278fedf4c4837b686","gift":{"type":"percentual","value":11000},"expiration_date":"2021-10-30T02:00:00.000Z","status":"D"},{"friendlyCode":"VALUE31999","description":"","id":"5fa2c1a878fedf4c4837b689","gift":{"type":"value","value":31999},"expiration_date":"2021-10-30T02:00:00.000Z","status":"D"},{"friendlyCode":"TESTEJERONIMO","description":"","id":"5fa2de8678fedf4c4837b697","gift":{"type":"value","value":1000},"expiration_date":"2021-11-04T03:00:00.000Z","status":"D"},{"friendlyCode":"MADERORICARDO","description":"","id":"5fa2e13078fedf4c4837b699","gift":{"type":"value","value":1000},"expiration_date":"2021-11-04T03:00:00.000Z","status":"D"},{"friendlyCode":"TESTECODE2","description":"TESTECODE2","id":"5fabf641a6a87516ba3bd466","gift":{"type":"value","value":400},"expiration_date":"2020-11-13T23:00:00.000Z","status":"D"},{"friendlyCode":"BRASIL10","description":"BRASIL10","id":"5fabf663a6a87516ba3bd467","gift":{"type":"value","value":1000},"expiration_date":"2020-11-13T17:00:00.000Z","status":"D"}],"nextsExpiringCashback":{"expiresTomorrow":0,"expiresInNextWeek":0}}}';

        if (empty($curl_response)){
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }
        else {
            $result = json_decode($curl_response, true);
            if (empty($result)) throw new \Exception($curl_response);
            $result['fidelidade']='4ALL';
            if (array_key_exists('success', $result)){
                if ($method === 'user' || $result['success']){
                    return $result;
                }
                else {
                    throw new \Exception($result['message']);
                }
            }
            else {
                throw new \Exception($result['error']);
            }
        }
    }

    private function callFidelityUDS($method, $params){
        self::Log('callFidelityUDS');
        $URL = $this->sessionHandler->getUserInfo()['CDURLWSFIDUDS'];
        $URL = rtrim($URL, '/') . "/";
        $URL .= $method;
        $curl = curl_init();
        self::Log($URL);
        self::Log(print_r($params,true));
        curl_setopt($curl, CURLOPT_URL, $URL);
        if ($method == 'getSaldo'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));
            $xml = new \SimpleXMLIterator("<?xml version=\"1.0\" encoding=\"utf-8\"?><root></root>");
            $xml->addChild('cpfCli',$params['cpf']);
            self::Log($xml->asXML());
            curl_setopt($curl, CURLOPT_POSTFIELDS, $xml->asXML());
        } else if ($method == 'sendSale'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, self::ArrayToXml(json_decode($params, true)));
        } else if ($method === 'cancelSale'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, self::ArrayToXml(json_decode($params, true)));
        } else if ($method === 'sendOrder'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, self::ArrayToXml(json_decode($params, true)));
        } else if ($method === 'deleteOrder'){
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/xml'));
            curl_setopt($curl, CURLOPT_POSTFIELDS, self::ArrayToXml(json_decode($params, true)));
        }

        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_ENCODING,'');
        curl_setopt($curl, CURLOPT_MAXREDIRS,10);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_HTTP_VERSION , CURL_HTTP_VERSION_1_1);

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 20);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 20);

        self::Log('curl_exec');
        $curl_response = curl_exec($curl);
        self::Log('curl_response: '.$curl_response);
        $err    = curl_errno($curl);
        $errmsg = curl_error($curl);
        $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        self::Log('httpcode: '.$httpcode);
        curl_close($curl);

        if ($err){
            self::Log('err: '.$err);
            self::Log('errmsg: '.$errmsg);

            if ($err === 28){
                throw new \Exception("<br>N&atilde;o foi poss&iacute;vel estabelecer uma conex&atilde;o com o servidor.");
            }
            else {
                throw new \Exception("<br>Ocorreu um erro na chamada com o servidor:<br><br>" . $errmsg);
            }
        }
        if ($httpcode ==='404' || $httpcode === 404){
            self::Log('Erro 404');
            throw new \Exception("<br>Erro 404 Page Not Found.");
        }
        if (empty($curl_response)){
            self::Log('empty curl_response');
            throw new \Exception("<br>N&atilde;o houve retorno do servidor.");
        }
        else {
            $result = self::XmlToJson($curl_response);
            $result = json_decode($result, true);
            if (empty($result)) throw new \Exception($curl_response);
            $result['fidelidade']='UDS';

            if (array_key_exists('success', $result)){
                self::Log('array_key_exists: success');
                if($result['success'] === true || $result['success'] === 'true'){
                    $result['success'] = true;
                    if ($method == 'getSaldo' ){
                        $result['userInfo']['balance'] = $result['saldoDisp'];
                        $result['userInfo']['cpf'] = $result['cpfCli'];
                        $result['userInfo']['name'] = $result['nomeCompleto'];
                    }
                    if (array_key_exists('bloqueado', $result)){
                        if($result['bloqueado'] === true || $result['bloqueado'] === 'true'){
                            $result['bloqueado'] = true;
                        } else {
                            $result['bloqueado'] = false;
                        }
                    } else {
                        $result['bloqueado'] = false;
                    }
                    self::Log(print_r($result,true));
                    self::Log('');
                    return $result;
                }
                else {
                    if(isset($result['message'])){
                        self::Log('message: '.$result['message']);
                        self::Log('');
                        throw new \Exception($result['message']);
                    } else if(isset($result['error'])){
                        self::Log('error: '.$result['error']);
                        self::Log('');
                        throw new \Exception($result['error']);
                    } else {
                        self::Log('Erro na consulta fidelidade');
                        self::Log('');
                        throw new \Exception("Erro na consulta fidelidade");
                    }
                }
            }
            else {
                self::Log('array_key_NOT_exists: success');
                if(isset($result['error'])){
                    self::Log('error: '.$result['error']);
                    self::Log('');
                    throw new \Exception($result['error']);
                } else if(isset($result['message'])){
                    self::Log('message: '.$result['message']);
                    self::Log('');
                    throw new \Exception($result['message']);
                } else {
                    self::Log(print_r($result,true));
                    self::Log('');
                    throw new \Exception("Erro na consulta fidelidade");
                }
            }
        }
    }

    public function isXml($string){
        return substr($string, 0, 5) == "<?xml";
    }

    public function XmlToJson ($xml) {
        self::Log('XmlToJson');
        $xml = str_replace(array("\n", "\r", "\t"), '', $xml);
        $xml = trim(str_replace('"', "'", $xml));
        $simpleXml = simplexml_load_string($xml);
        $json = json_encode($simpleXml);
        self::Log($json);
        return $json;
    }

    function ArrayToXml($array, $xml = null){
        if($xml == null){
            $xml = new SimpleXMLElement('<root/>');
        }        
        foreach ($array as $key => $value) {
            if(is_int($key) && is_array($value)){
                self::ArrayToXml($value, $xml);
            } else {
                if(is_int($key)){
                    if($key===0){
                        $key = '0';
                    } else {
                        $key = strval($key);
                    }
                }
                if(is_bool($key)){
                    if($key){
                        $key = 'true';
                    }else {
                        $key = 'false';
                    }
                }
                if(is_array($value)){
                    $label = $xml->addChild($key);
                    self::ArrayToXml($value, $label);
                }
                else {
                    $xml->addChild($key, $value);
                }
            }
        }
        self::Log($xml->asXml());
        return $xml->asXml();
    }

    public function Log($texto){
        $paths = ['LOG/', 'fidelidadeUDS/', (new \DateTime())->format('d-m-Y')];
        $logVendaPath = dirname(__DIR__) . '/../../../../../../';
        foreach ($paths as $path){
            $logVendaPath = $logVendaPath . $path;
            if (!is_dir($logVendaPath)){
                mkdir($logVendaPath, 0700);
            }
        }
        $logVendaPath = $logVendaPath .'/'. (new \DateTime())->format('d-m-Y') . '.txt';
        $horaAtual = (new \DateTime())->format('d/m/Y H:i:s');
        file_put_contents($logVendaPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
    }
}